package cn.fateverse.query.mapper;

import cn.fateverse.query.entity.PortalMapping;
import org.apache.ibatis.annotations.MapKey;

import java.util.List;
import java.util.Map;

/**
 * @author Clay
 * @date 2024/4/15 15:20
 */
public interface PortalMappingMapper {

    /**
     * 根据portalId查询
     *
     * @param portalId 接口模块id
     * @return 映射key
     */
    List<PortalMapping> selectByPortalId(Long portalId);

    /**
     * 批量插入
     *
     * @param list 映射关系
     * @return 插入数量
     */
    Integer insertBatch(List<PortalMapping> list);

    /**
     * 根据portalId删除
     *
     * @param portalId 接口模块id
     * @return 删除数量
     */
    Integer deleteByPortalId(Long portalId);


    List<PortalMapping> selectByPortalIds(List<Long> portalIds);
}
